<?php

namespace App\Http\Controllers\V1;

use App\Exceptions\ServerRunTimeException;
use App\Services\AuthService;
use App\Services\YyBao\CustomersService;
use App\Support\Traits\Helpers;
use Illuminate\Auth\AuthenticationException;
use Illuminate\Http\Request;
use Jiannei\Response\Laravel\Support\Facades\Response;
use Jiannei\Response\Laravel\Support\Traits\ExceptionTrait;
use Laravel\Lumen\Routing\Controller as BaseController;
use OpenApi\Annotations\Info;
use OpenApi\Annotations\Property;
use OpenApi\Annotations\Schema;

/**
 *
 * @Info(
 *     version="1.0",
 *     title="研易宝"
 * )
 *
 * @Schema(
 *     schema="ApiResponse",
 *     type="object",
 *     title="接口返回统一响应实体",
 *     @Property(property="status", type="string", description="响应结果提示"),
 *     @Property(
 *         property="code",
 *         type="string",
 *         description="响应代码"
 *     ),
 *     @Property(property="message", type="string", description="响应结果提示"),
 *     @Property(property="data", type="object", description="响应结果数据集"),
 *     @Property(property="error", type="object", description="响应错误结果数据集")
 * )
 *
 */
abstract class Controller extends BaseController
{
    use Helpers, ExceptionTrait;

    public function getRequestRulesInputs(Request $request): array
    {
        $inputs = $request->request->all();
        $rules = $request->rules();

        foreach($inputs as $key => $in){
            if(!empty($rules) && !in_array($key,array_keys($rules))){
                unset($inputs[$key]);
            }
            $inputs[$key] = $this->stripHtml($in);
        }
        $inputs['platform_source'] = current($this->getUriPath());
        return $inputs;
    }

    /**
     * request 参数验证
     * @param $request
     * @param $message
     * @return bool|\Illuminate\Http\JsonResponse
     */
    protected function validateRequest($request, $message){

        $validateArr = [];
        collect($message)->map(function($value,$key) use (&$validateArr){
            $keys = explode('.',$key);
            $validateArr[$keys[0]] = $keys[1];
        });

        $validator = Validator($request->all(), $validateArr, $message);

        if ($validator->fails()) {
            foreach($validator->errors()->getMessages() as $error) {
                return Response::fail($error[0]);
            }
        }
        return true;
    }

    public function getGuardName()
    {
        return app()->request->headers->get('guardName');
    }

    protected function returnErrorExecptionResponse(\Exception $exception, string $message = ''): \Illuminate\Http\JsonResponse
    {
        if($exception instanceof AuthenticationException){
            return Response::fail($exception->getMessage(),401);
        }
        if($exception->getCode() == 500 || $exception->getCode() == 0){
            return Response::fail($exception->getMessage(),500);
        }else{
            return Response::fail($message == '' ? $exception->getMessage() : $message,$exception->getCode());
        }
    }

    public function setService($service)
    {
        $this->service = $service;
    }

    public function getService()
    {
        return $this->service;
    }

    public function getUriPath()
    {
        $urls = explode('/',app()->request->url);
        return array_values(array_filter($urls));
    }
}
