<?php

/*
 * This file is part of the Jiannei/lumen-api-starter.
 *
 * (c) Jiannei <longjian.huang@foxmail.com>
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

namespace App\Services;

use App\Repositories\Contracts\PlatformCustomerRepository;
use App\Repositories\Transformers\CustomersTransformer;
use App\Repositories\Transformers\OtherCustomersTransformer;
use App\Rhawn\Repositories\Contracts\BhuaCustomerRepository;
use App\Rhawn\Repositories\Contracts\Chemsite\ChemsiteCompanyRepository;
use App\Rhawn\Repositories\Contracts\RhawnCustomerRepository;
use App\Rhawn\Services\BhuaCustomerService;
use App\Rhawn\Services\Chemsite\ChemsiteCompanyService;
use App\Rhawn\Services\RhawnCustomerService;
use App\Support\Facades\SimpleLogs;
use App\Support\Traits\Helpers;

class CustomerService
{
    use Helpers;

    public function __construct(PlatformCustomerRepository $platformCustomerRepository)
    {
        $this->platformCustomerRepository = $platformCustomerRepository;
        $this->rhawnCustomerService = (new RhawnCustomerService(app(RhawnCustomerRepository::class)));
        $this->bhuaCustomerService = (new BhuaCustomerService(app(BhuaCustomerRepository::class)));
        $this->chemsiteCustomerService = (new ChemsiteCompanyService(app(ChemsiteCompanyRepository::class)));
    }

    /**
     * 创建api会员信息
     * @param $requestParams
     * @throws \Exception
     */
    public function createPlatformCustomer($requestParams)
    {
        try{
            switch($requestParams['customer_type']){
                case 'rhawn' :
                    $customer = $this->rhawnCustomerService->getRhawnCustomerThroughtCusCode($requestParams['customer_code']);
                    break;
                case 'bh' :
                    $customer = $this->bhuaCustomerService->getBhuaCustomerThroughtCusCode($requestParams['customer_code']);
                    break;
                case 'chemsite' :
                    $customer = $this->chemsiteCustomerService->getCompanyThroughCusCode($requestParams['customer_code']);
                    break;
            }
            if(!$customer){
                throw new \Exception('没有找到该客户',500);
            }

            $platformCustomer = $this->platformCustomerRepository->findWhere([
                'cus_number' => $customer['cus_no'],
                'cus_type' => $requestParams['customer_type']
            ])->toArray();
            if($platformCustomer){
                throw new \Exception('平台用户已存在，无需重复注册',500);
            }


            $newCustomer = [];
            $newCustomer['cus_id'] = $customer['cus_id'];
            $newCustomer['cus_number'] = $requestParams['customer_type'] == 'chemsite' ? $customer['com_cus_no'] : $customer['cus_no'];
            $newCustomer['company_name'] = $requestParams['customer_type'] == 'chemsite' ? $customer['com_name'] : $requestParams['customer_company'];
            $newCustomer['token'] = $this->randomFromDevice(16);
            $newCustomer['cus_type'] = $requestParams['customer_type'];
            $newCustomer['platform_params'] = isset($requestParams['platform_params']) ? json_encode($requestParams['platform_params'],JSON_UNESCAPED_UNICODE) : '';
            $newCustomer['status'] = isset($requestParams['status']) ? $requestParams['status'] : 1;

            SimpleLogs::writeLog($newCustomer,__CLASS__.':createCustomer => params');
            $platformCustomer = $this->platformCustomerRepository->create($newCustomer)->toArray();
            if($platformCustomer){
                return $platformCustomer;
            }
            return false;

        }catch(\Throwable $exception){
            SimpleLogs::writeLog($exception->getMessage(),__CLASS__.':createCustomer','error');
            throw $exception;
        }
    }

    /**
     * 获取客户信息
     * @param $requestParams
     * @return array
     * @throws \Throwable
     */
    public function getList($requestParams)
    {
        try{
            $offset = !isset($requestParams['page_size']) || empty($requestParams['page_size']) ? 100 : $requestParams['page_size'];
            if($offset > 100){
                $offset = 100;
            }
            $pageNo = !isset($requestParams['page_no']) || empty($requestParams['page_no']) ? 1 : $requestParams['page_no'];
            $limit = $pageNo == 1 ? 0 : $pageNo * $offset;

            $customerRepository = app(PlatformCustomerRepository::class);

            $where = [];
            if(isset($requestParams['customer_no']) && !empty($requestParams['customer_no'])){
                $where = [
                    'cus_number' => $requestParams['customer_no']
                ];
            }
            $customerList = $customerRepository->getCustomerList($where,$limit,$offset);

            //获取订单量总数
            $total = $customerRepository->count($where);

            $dataReturn = [];
            $dataReturn['data'] = app(CustomersTransformer::class)->transform($customerList);
            $dataReturn['total_page'] = ceil($total / $offset);
            $dataReturn['page_no'] = $pageNo;

            return $dataReturn;
        }catch(\throwable $exception){
            simplelogs::writelog($exception->getmessage(),__class__.':getList','error');
            throw $exception;
        }
    }

    /**
     * 获取平台用户信息
     * @param $requestParams
     */
    public function getPlatformCustomer($requestParams)
    {
        $platformCustomer = $this->platformCustomerRepository->findWhere($requestParams);
        if(!$platformCustomer){
            throw new \Exception('平台用户不存在',500);
        }
        return $platformCustomer->first()->toArray();
    }

    /**
     * 更新平台用户token
     * @param $requestParams
     * @throws \Throwable
     */
    public function refreshCustomerToken($requestParams)
    {
        try{
            $platformCustomer = $this->platformCustomerRepository->find($requestParams['platform_customer_id']);
            if(!$platformCustomer){
                throw new \Exception('平台用户不存在',500);
            }
            $newToken = $this->randomFromDevice(16);
            $this->platformCustomerRepository->update([
                'token' => $newToken
            ],$platformCustomer['id']);

            return $newToken;

        }catch(\Throwable $exception){
            SimpleLogs::writeLog($exception->getMessage(),__CLASS__.':refreshCustomerToken','error');
            throw $exception;
        }
    }

    /**
     * 根据token获取平台用户信息
     * @param $token
     * @return false|mixed
     * @throws \Throwable
     */
    public function getCustomerInfoByToken($token)
    {
        try{
            $platformCustomer = $this->platformCustomerRepository->findWhere([
                'token' => $token
            ]);
            if(!$platformCustomer){
                throw new \Exception('平台用户不存在',500);
            }
            return current($platformCustomer->toArray());
        }catch(\Throwable $exception){
            SimpleLogs::writeLog($exception->getMessage(),__CLASS__.':refreshCustomerToken','error');
            throw $exception;
        }
    }

    public function searchOtherCustomerInfo($requestParams)
    {
        try{
            $offset = !isset($requestParams['page_size']) || empty($requestParams['page_size']) ? 100 : $requestParams['page_size'];
            if($offset > 100){
                $offset = 100;
            }
            $pageNo = !isset($requestParams['page_no']) || empty($requestParams['page_no']) ? 1 : $requestParams['page_no'];
            $limit = $pageNo == 1 ? 0 : $pageNo * $offset;

            $customerService = null;
            switch($requestParams['customer_type']){
                case 'bh' : $customerService = app(BhuaCustomerService::class);break;
                case 'rhawn' : $customerService = app(RhawnCustomerService::class);break;
                case 'chemsite' : $customerService = app(ChemsiteCompanyService::class);break;
            }

            if($customerService == null){
                throw new \Exception('客户类型错误',500);
            }

            $customerList = $customerService->getCustomerListByCusCode($requestParams['customer_no'],$limit,$offset);

            //获取订单量总数
            $total = $customerService->customerRepository->count(['cus_no' => $requestParams['customer_no']]);

            $dataReturn = [];
            $dataReturn['data'] = app(OtherCustomersTransformer::class)->transform($customerList);
            $dataReturn['total_page'] = ceil($total / $offset);
            $dataReturn['page_no'] = $pageNo;

            return $dataReturn;
        }catch(\throwable $exception){
            simplelogs::writelog($exception->getmessage(),__class__.':getList','error');
            throw $exception;
        }
    }

    /**
     * 获取token
     * @param $requestParams
     * @return array
     * @throws \Exception
     */
    public function getToken($requestParams)
    {
        $platformCustomer = $this->platformCustomerRepository->findWhere([
            'token' => $requestParams['customer_token']
        ]);
        if(!$platformCustomer->toArray()){
            throw new \Exception('token无效',500);
        }
        $platformCustomer = current($platformCustomer->toArray());
        if($platformCustomer['cus_type'] == 'chemsite'){
            $token['company_token'] = $requestParams['customer_token'];
            $token['cus_number'] = $requestParams['cus_number'] ?? '';
        }else{
            $token['customer_token'] = $requestParams['customer_token'];
        }
        $newToken = base64_encode(implode(',', $token));

        $expTimestamp = strtotime(date('Y-m-d H:i:s', strtotime('+1 day')));
        $jwtToken = $this->generateToken($newToken);

        return [
            'token' => $jwtToken,
            'ttl' => $expTimestamp
        ];
    }


}
